@@ -34,7 +34,6 @@
 #include <gtk/gtk.h>
 
 #include "gpa.h"   
-#include "gpapastrings.h"
 
 #include "gtktools.h"
 #include "gpgmetools.h"
@@ -271,12 +270,30 @@ file_created_cb (GpaFileOperation *op, gpa_file_item_t item, gpointer data)
   if (! suc)
     {
       gchar *str;
-      str = g_strdup_printf ("Error in operation result:\n"
-			     "No valid UTF-8 at position %i.",
+      gsize *len;
+
+      str = g_strdup_printf ("No valid UTF-8 encoding at position %i.\n"
+                             "Assuming Latin-1 encoding instead.",
 			     ((int) (end - item->direct_out)));
-      gpa_window_error (str, GTK_WIDGET (clipboard));
+      gpa_window_message (str, GTK_WIDGET (clipboard));
       g_free (str);
-      return;
+
+      str = g_convert (item->direct_out, item->direct_out_len,
+                       "UTF-8", "ISO-8859-1",
+                       NULL, &len, NULL);
+      if (str)
+        {
+#ifdef G_OS_WIN32
+          dos_to_unix (str, &item->direct_out_len);
+#endif
+          
+          gtk_text_buffer_set_text (clipboard->text_buffer, str, len);
+          g_free (str);
+          return;
+        }
+      gpa_window_error ("Error converting Latin-1 to UTF-8",
+                        GTK_WIDGET (clipboard));
+      /* Enough warnings: Try to show even with invalid encoding.  */
     }
 
 #ifdef G_OS_WIN32
@@ -831,6 +848,7 @@ clipboard_action_new (GpaClipboard *clipboard,
     "      <menuitem action='WindowsKeyringEditor'/>"
     "      <menuitem action='WindowsFileManager'/>"
     "      <menuitem action='WindowsClipboard'/>"
+    "      <menuitem action='WindowsCardManager'/>"
     "    </menu>"
     "    <menu action='Help'>"
 #if 0
@@ -861,6 +879,7 @@ clipboard_action_new (GpaClipboard *clipboard,
     "    <separator/>"
     "    <toolitem action='WindowsKeyringEditor'/>"
     "    <toolitem action='WindowsFileManager'/>"
+    "    <toolitem action='WindowsCardManager'/>"
 #if 0
     "    <toolitem action='HelpContents'/>"
 #endif
@@ -874,6 +893,7 @@ clipboard_action_new (GpaClipboard *clipboard,
   GError *error;
 
   action_group = gtk_action_group_new ("MenuActions");
+  gtk_action_group_set_translation_domain (action_group, PACKAGE);
   gtk_action_group_add_actions (action_group, entries, G_N_ELEMENTS (entries),
 				clipboard);
   gtk_action_group_add_actions (action_group, gpa_help_menu_action_entries,
@@ -902,6 +922,8 @@ clipboard_action_new (GpaClipboard *clipboard,
   g_object_set (action, "short_label", _("Keyring"), NULL);
   action = gtk_action_group_get_action (action_group, "WindowsFileManager");
   g_object_set (action, "short_label", _("Files"), NULL);
+  action = gtk_action_group_get_action (action_group, "WindowsCardManager");
+  g_object_set (action, "short_label", _("Card"), NULL);
 
   /* Take care of sensitiveness of widgets.  */
   action = gtk_action_group_get_action (action_group, "EditCut");
@@ -1098,7 +1120,8 @@ gpa_clipboard_get_instance (void)
   return GTK_WIDGET (instance);
 }
 
-gboolean gpa_clipboard_is_open (void)
+gboolean
+gpa_clipboard_is_open (void)
 {
   return (instance != NULL);
 }
