@@ -41,10 +41,10 @@
 #include "gpawidgets.h"
 
 /* Internal functions */
-static void gpa_file_encrypt_operation_done_error_cb (GpaContext *context, 
+static void gpa_file_encrypt_operation_done_error_cb (GpaContext *context,
 						      gpg_error_t err,
 						      GpaFileEncryptOperation *op);
-static void gpa_file_encrypt_operation_done_cb (GpaContext *context, 
+static void gpa_file_encrypt_operation_done_cb (GpaContext *context,
 						gpg_error_t err,
 						GpaFileEncryptOperation *op);
 static void gpa_file_encrypt_operation_response_cb (GtkDialog *dialog,
@@ -68,7 +68,7 @@ gpa_file_encrypt_operation_get_property (GObject *object, guint prop_id,
 					 GValue *value, GParamSpec *pspec)
 {
   GpaFileEncryptOperation *op = GPA_FILE_ENCRYPT_OPERATION (object);
-  
+
   switch (prop_id)
     {
     case PROP_FORCE_ARMOR:
@@ -100,7 +100,7 @@ gpa_file_encrypt_operation_set_property (GObject *object, guint prop_id,
 
 static void
 gpa_file_encrypt_operation_finalize (GObject *object)
-{  
+{
   GpaFileEncryptOperation *op = GPA_FILE_ENCRYPT_OPERATION (object);
 
   /* FIXME: The use of RSET is messed up.  There is no clear concept
@@ -126,7 +126,7 @@ gpa_file_encrypt_operation_init (GpaFileEncryptOperation *op)
 }
 
 static GObject*
-gpa_file_encrypt_operation_constructor 
+gpa_file_encrypt_operation_constructor
 	(GType type,
          guint n_construct_properties,
          GObjectConstructParam *construct_properties)
@@ -163,7 +163,7 @@ static void
 gpa_file_encrypt_operation_class_init (GpaFileEncryptOperationClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
-  
+
   parent_class = g_type_class_peek_parent (klass);
 
   object_class->constructor = gpa_file_encrypt_operation_constructor;
@@ -184,7 +184,7 @@ GType
 gpa_file_encrypt_operation_get_type (void)
 {
   static GType file_encrypt_operation_type = 0;
-  
+
   if (!file_encrypt_operation_type)
     {
       static const GTypeInfo file_encrypt_operation_info =
@@ -199,12 +199,12 @@ gpa_file_encrypt_operation_get_type (void)
         0,              /* n_preallocs */
         (GInstanceInitFunc) gpa_file_encrypt_operation_init,
       };
-      
-      file_encrypt_operation_type = g_type_register_static 
+
+      file_encrypt_operation_type = g_type_register_static
 	(GPA_FILE_OPERATION_TYPE, "GpaFileEncryptOperation",
 	 &file_encrypt_operation_info, 0);
     }
-  
+
   return file_encrypt_operation_type;
 }
 
@@ -215,7 +215,7 @@ gpa_file_encrypt_operation_new (GtkWidget *window, GList *files,
 				gboolean force_armor)
 {
   GpaFileEncryptOperation *op;
-  
+
   op = g_object_new (GPA_FILE_ENCRYPT_OPERATION_TYPE,
 		     "window", window,
 		     "input_files", files,
@@ -231,7 +231,7 @@ gpa_file_encrypt_sign_operation_new (GtkWidget *window, GList *files,
 				     gboolean force_armor)
 {
   GpaFileEncryptOperation *op;
-  
+
   op = g_object_new (GPA_FILE_ENCRYPT_OPERATION_TYPE,
 		     "window", window,
 		     "input_files", files,
@@ -249,7 +249,7 @@ GpaFileEncryptOperation*
 gpa_file_encrypt_operation_new_for_server (GList *files, void *server_ctx)
 {
   GpaFileEncryptOperation *op;
-  
+
   op = g_object_new (GPA_FILE_ENCRYPT_OPERATION_TYPE,
 		     "input_files", files,
                      "server-ctx", server_ctx,
@@ -299,7 +299,7 @@ gpa_file_encrypt_operation_start (GpaFileEncryptOperation *op,
 	  gpa_gpgme_warning (err);
 	  return err;
 	}
-      
+
       err = gpgme_data_new (&op->cipher);
       if (err)
 	{
@@ -312,11 +312,11 @@ gpa_file_encrypt_operation_start (GpaFileEncryptOperation *op,
   else
     {
       gchar *plain_filename = file_item->filename_in;
-  
-      file_item->filename_out = destination_filename 
+
+      file_item->filename_out = destination_filename
 	(plain_filename, gpgme_get_armor (GPA_OPERATION (op)->context->ctx));
       /* Open the files */
-      op->plain_fd = gpa_open_input (plain_filename, &op->plain, 
+      op->plain_fd = gpa_open_input (plain_filename, &op->plain,
 				     GPA_OPERATION (op)->window);
       if (op->plain_fd == -1)
 	/* FIXME: Error value.  */
@@ -337,7 +337,7 @@ gpa_file_encrypt_operation_start (GpaFileEncryptOperation *op,
   /* Start the operation.  */
   /* Always trust keys, because any untrusted keys were already
      confirmed by the user.  */
-  if (gpa_file_encrypt_dialog_get_sign 
+  if (gpa_file_encrypt_dialog_get_sign
       (GPA_FILE_ENCRYPT_DIALOG (op->encrypt_dialog)))
     err = gpgme_op_encrypt_sign_start (GPA_OPERATION (op)->context->ctx,
 				       op->rset, GPGME_ENCRYPT_ALWAYS_TRUST,
@@ -365,7 +365,7 @@ gpa_file_encrypt_operation_start (GpaFileEncryptOperation *op,
 
   /* Show and update the progress dialog.  */
   gtk_widget_show_all (GPA_FILE_OPERATION (op)->progress_dialog);
-  gpa_progress_dialog_set_label (GPA_PROGRESS_DIALOG 
+  gpa_progress_dialog_set_label (GPA_PROGRESS_DIALOG
 				 (GPA_FILE_OPERATION (op)->progress_dialog),
 				 file_item->direct_name
 				 ? file_item->direct_name
@@ -392,7 +392,7 @@ gpa_file_encrypt_operation_next (GpaFileEncryptOperation *op)
 
 
 static void
-gpa_file_encrypt_operation_done_cb (GpaContext *context, 
+gpa_file_encrypt_operation_done_cb (GpaContext *context,
 				    gpg_error_t err,
 				    GpaFileEncryptOperation *op)
 {
@@ -434,7 +434,7 @@ gpa_file_encrypt_operation_done_cb (GpaContext *context,
   op->cipher_fd = -1;
   gtk_widget_hide (GPA_FILE_OPERATION (op)->progress_dialog);
 
-  if (err) 
+  if (err)
     {
       if (! file_item->direct_in)
 	{
@@ -452,7 +452,7 @@ gpa_file_encrypt_operation_done_cb (GpaContext *context,
       g_signal_emit_by_name (GPA_OPERATION (op), "created_file", file_item);
 
       /* Go to the next file in the list and encrypt it */
-      GPA_FILE_OPERATION (op)->current = g_list_next 
+      GPA_FILE_OPERATION (op)->current = g_list_next
 	(GPA_FILE_OPERATION (op)->current);
       gpa_file_encrypt_operation_next (op);
     }
@@ -474,7 +474,7 @@ ignore_key_trust (gpgme_key_t key, GtkWidget *parent)
   GtkResponseType response;
 
   dialog = gtk_dialog_new_with_buttons (_("Unknown Key"), GTK_WINDOW(parent),
-					GTK_DIALOG_MODAL, 
+					GTK_DIALOG_MODAL,
 					_("_Yes"), GTK_RESPONSE_YES,
 					_("_No"), GTK_RESPONSE_NO,
 					NULL);
@@ -500,7 +500,7 @@ ignore_key_trust (gpgme_key_t key, GtkWidget *parent)
   gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
   gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, TRUE, 5);
   label = gtk_label_new (NULL);
-  gtk_label_set_markup (GTK_LABEL (label), 
+  gtk_label_set_markup (GTK_LABEL (label),
 			_("Do you <b>really</b> want to use this key?"));
   gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
   gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, TRUE, 5);
@@ -523,7 +523,7 @@ revoked_key (gpgme_key_t key, GtkWidget *parent)
   GtkWidget *image;
 
   dialog = gtk_dialog_new_with_buttons (_("Revoked Key"), GTK_WINDOW(parent),
-					GTK_DIALOG_MODAL, 
+					GTK_DIALOG_MODAL,
 					_("_Close"), GTK_RESPONSE_CLOSE,
 					NULL);
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
@@ -537,7 +537,7 @@ revoked_key (gpgme_key_t key, GtkWidget *parent)
   gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
   gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox);
 
-  label = gtk_label_new (_("The following key has been revoked by it's owner:"));
+  label = gtk_label_new (_("The following key has been revoked by its owner:"));
   gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
   gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, TRUE, 5);
   key_info = gpa_key_info_new (key);
@@ -563,7 +563,7 @@ expired_key (gpgme_key_t key, GtkWidget *parent)
   gchar *message;
 
   dialog = gtk_dialog_new_with_buttons (_("Revoked Key"), GTK_WINDOW(parent),
-					GTK_DIALOG_MODAL, 
+					GTK_DIALOG_MODAL,
 					_("_Close"), GTK_RESPONSE_CLOSE,
 					NULL);
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
@@ -578,7 +578,7 @@ expired_key (gpgme_key_t key, GtkWidget *parent)
   gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox);
 
   message = g_strdup_printf (_("The following key expired on %s:"),
-                             gpa_expiry_date_string 
+                             gpa_expiry_date_string
                              (key->subkeys->expires));
   label = gtk_label_new (message);
   gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
@@ -615,11 +615,11 @@ set_recipients (GpaFileEncryptOperation *op, GList *recipients)
         {
           /* Should not happen either because the selection dialog
              should have not allowed to select differet keys.  */
-          gpa_window_error 
+          gpa_window_error
             (_("The selected certificates are not all of the same type."
                " That is, you mixed OpenPGP and X.509 certificates."
                " Please make sure to select only certificates of the"
-               " same type."), 
+               " same type."),
              GPA_OPERATION (op)->window);
           return FALSE;
         }
@@ -648,7 +648,7 @@ set_recipients (GpaFileEncryptOperation *op, GList *recipients)
          valid becuase the backend will chekc this.  FIXME: It would
          be better to ask the backend to check the validity of the key
          instead of letting it fail later. */
-      else if (valid == GPGME_VALIDITY_FULL 
+      else if (valid == GPGME_VALIDITY_FULL
                || valid == GPGME_VALIDITY_ULTIMATE
                || key->protocol == GPGME_PROTOCOL_CMS)
 	{
@@ -715,17 +715,17 @@ static void gpa_file_encrypt_operation_response_cb (GtkDialog *dialog,
   GpaFileEncryptOperation *op = user_data;
 
   gtk_widget_hide (GTK_WIDGET (dialog));
-  
+
   if (response == GTK_RESPONSE_OK)
     {
       gboolean success = TRUE;
-      gboolean armor = gpa_file_encrypt_dialog_get_armor 
+      gboolean armor = gpa_file_encrypt_dialog_get_armor
 	(GPA_FILE_ENCRYPT_DIALOG (op->encrypt_dialog));
-      GList *signers = gpa_file_encrypt_dialog_signers 
+      GList *signers = gpa_file_encrypt_dialog_signers
 	(GPA_FILE_ENCRYPT_DIALOG (op->encrypt_dialog));
       GList *recipients = gpa_file_encrypt_dialog_recipients
 	(GPA_FILE_ENCRYPT_DIALOG (op->encrypt_dialog));
-      
+
       /* Set the armor value */
       gpgme_set_armor (GPA_OPERATION (op)->context->ctx, armor);
       /* Set the signers for the context.  */
