@@ -1,6 +1,6 @@
 /* gpgmetools.h - Additional gpgme support functions for GPA.
    Copyright (C) 2002 Miguel Coca.
-   Copyright (C) 2005, 2008 g10 Code GmbH.
+   Copyright (C) 2005, 2008, 2009 g10 Code GmbH.
 
    This file is part of GPA
 
@@ -25,6 +25,8 @@
 
 #include <errno.h>
 #include <ctype.h>
+#include <stdarg.h>
+
 #include "gpa.h"
 #include "gtktools.h"
 #include "gpgmetools.h"
@@ -52,6 +54,17 @@
 #endif
 #endif
 
+/* Helper to strip the path from a source file name.  This helps to
+   avoid showing long filenames in case of VPATH builds.  */
+static const char *
+strip_path (const char *file)
+{
+  const char *s = strrchr (file, '/');
+  return s? s+1:file;
+}
+
+
+
 /* Report an unexpected error in GPGME and quit the application.  */
 void
 _gpa_gpgme_error (gpg_error_t err, const char *file, int line)
@@ -60,7 +73,8 @@ _gpa_gpgme_error (gpg_error_t err, const char *file, int line)
                                       "(invoked from file %s, line %i):\n\n"
                                       "\t%s\n\n"
                                       "The application will be terminated"),
-                                    file, line, gpgme_strerror (err));
+                                    strip_path (file), line,
+                                    gpgme_strerror (err));
   gpa_window_error (message, NULL);
   g_free (message);
   exit (EXIT_FAILURE);
@@ -68,18 +82,39 @@ _gpa_gpgme_error (gpg_error_t err, const char *file, int line)
 
 
 void
-gpa_gpgme_warning (gpg_error_t err)
+gpa_gpgme_warning_ext (gpg_error_t err, const char *desc)
 {
-  gchar *message = g_strdup_printf (_("The GPGME library returned an unexpected\n"
-				      "error. The error was:\n\n"
-                                      "\t%s\n\n"
-                                      "This is probably a bug in GPA.\n"
-				      "GPA will now try to recover from this error."),
-                                    gpgme_strerror (err));
+  char *argbuf = NULL;
+  const char *arg;
+  char *message;
+
+  if (desc && (!err || gpg_err_code (err) == GPG_ERR_GENERAL))
+    arg = desc;
+  else if (desc)
+    {
+      argbuf = g_strdup_printf ("%s (%s)", gpgme_strerror (err), desc);
+      arg = argbuf;
+    }
+  else
+    arg = gpgme_strerror (err);
+
+  message = g_strdup_printf
+    (_("The GPGME library returned an unexpected\n"
+       "error. The error was:\n\n"
+       "\t%s\n\n"
+       "This is probably a bug in GPA.\n"
+       "GPA will now try to recover from this error."),
+     arg);
+  g_free (argbuf);
   gpa_window_error (message, NULL);
   g_free (message);
 }
 
+void
+gpa_gpgme_warning (gpg_error_t err)
+{
+  gpa_gpgme_warning_ext (err, NULL);
+}
 
 /* Initialize a gpgme_ctx_t for use with GPA.  */
 gpgme_ctx_t
@@ -95,7 +130,7 @@ gpa_gpgme_new (void)
 
   if (! cms_hack)
     gpgme_set_passphrase_cb (ctx, gpa_passphrase_cb, NULL);
-  
+
   return ctx;
 }
 
@@ -133,9 +168,9 @@ check_overwriting (const char *filename, GtkWidget *parent)
   /* If the file exists, ask before overwriting.  */
   if (g_file_test (filename, G_FILE_TEST_EXISTS))
     {
-      GtkWidget *msgbox = gtk_message_dialog_new 
+      GtkWidget *msgbox = gtk_message_dialog_new
 	(GTK_WINDOW(parent), GTK_DIALOG_MODAL,
-	 GTK_MESSAGE_WARNING, GTK_BUTTONS_NONE, 
+	 GTK_MESSAGE_WARNING, GTK_BUTTONS_NONE,
 	 _("The file %s already exists.\n"
 	   "Do you want to overwrite it?"), filename);
       gtk_dialog_add_buttons (GTK_DIALOG (msgbox),
@@ -160,7 +195,7 @@ FILE *
 gpa_fopen (const char *filename, GtkWidget *parent)
 {
   FILE *target;
-  
+
   if (!check_overwriting (filename, parent))
     return NULL;
   target = g_fopen (filename, "w");
@@ -268,7 +303,7 @@ dos_to_unix (gchar *str, gsize *len)
      inserted).  */
   gchar *src;
   gchar *dst;
-  
+
   src = str;
   dst = str;
   while (*src)
@@ -291,7 +326,7 @@ dump_data_to_clipboard (gpgme_data_t data, GtkClipboard *clipboard)
   char buffer[128];
   int nread;
   gchar *text = NULL;
-  gint len = 0;
+  size_t len = 0;
 
   nread = gpgme_data_seek (data, 0, SEEK_SET);
   if (nread == -1)
@@ -315,7 +350,7 @@ dump_data_to_clipboard (gpgme_data_t data, GtkClipboard *clipboard)
   dos_to_unix (text, &len);
 #endif
 
-  gtk_clipboard_set_text (clipboard, text, len);
+  gtk_clipboard_set_text (clipboard, text, (int)len);
   g_free (text);
   return;
 }
@@ -325,80 +360,89 @@ dump_data_to_clipboard (gpgme_data_t data, GtkClipboard *clipboard)
    don't need worry about the user ID being UTF-8 as long as we are
    using GTK+2, because all user input is UTF-8 in it.  */
 static gchar *
-build_genkey_parms (GPAKeyGenParameters *params)
+build_genkey_parms (gpa_keygen_para_t *params)
 {
   gchar *string;
-  gchar *key_algo;
-  gchar *subkeys, *email, *comment, *expire;
+  const char *key_algo;
+  gchar *subkeys = NULL;
+  gchar *name = NULL;
+  gchar *email = NULL;
+  gchar *comment = NULL;
+  gchar *expire = NULL;
 
   /* Choose which keys and subkeys to generate.  */
   switch (params->algo)
     {
-    case GPA_KEYGEN_ALGO_DSA_ELGAMAL:
-      key_algo = "DSA";
-      subkeys = g_strdup_printf ("SubKey-Type: ELG-E\n"
-                                 "Subkey-Length: %i\n", params->keysize);
+    case GPA_KEYGEN_ALGO_RSA_RSA:
+      key_algo = "RSA";
+      subkeys = g_strdup_printf ("Subkey-Type: RSA\n"
+                                 "Subkey-Length: %d\n"
+                                 "Subkey-Usage: encrypt\n", params->keysize);
       break;
-    case GPA_KEYGEN_ALGO_DSA:
-      key_algo = "DSA";
-      subkeys = "";
+    case GPA_KEYGEN_ALGO_RSA_ELGAMAL:
+      key_algo = "RSA";
+      subkeys = g_strdup_printf ("Subkey-Type: ELG-E\n"
+                                 "Subkey-Length: %d\n"
+                                 "Subkey-Usage: encrypt\n", params->keysize);
       break;
     case GPA_KEYGEN_ALGO_RSA:
       key_algo = "RSA";
-      subkeys = "";
+      break;
+    case GPA_KEYGEN_ALGO_DSA_ELGAMAL:
+      key_algo = "DSA";
+      subkeys = g_strdup_printf ("Subkey-Type: ELG-E\n"
+                                 "Subkey-Length: %i\n"
+                                 "Subkey-Usage: encrypt\n", params->keysize);
+      break;
+    case GPA_KEYGEN_ALGO_DSA:
+      key_algo = "DSA";
       break;
     default:
       /* Can't happen */
       return NULL;
     }
 
-  /* Build the extra fields of the user ID if supplied by the user.  */
+  /* Construct the user ID.  */
+  if (params->name && params->name[0])
+    name = g_strdup_printf ("Name-Real: %s\n", params->name);
   if (params->email && params->email[0])
     email = g_strdup_printf ("Name-Email: %s\n", params->email);
-  else
-    email = "";
-
   if (params->comment && params->comment[0])
     comment = g_strdup_printf ("Name-Comment: %s\n", params->comment);
-  else
-    comment = "";
 
   /* Build the expiration date string if needed */
-  if (params->expiryDate)
-    {
-      expire = g_strdup_printf ("Expire-Date: %i-%02i-%02i\n", 
-                                g_date_get_year(params->expiryDate),
-                                g_date_get_month(params->expiryDate),
-                                g_date_get_day(params->expiryDate));
-    }
-  else if (params->interval)
-    expire = g_strdup_printf ("Expire-Date: %i%c\n", params->interval,
-			      params->unit);
-  else
-    expire = "";
+  if (g_date_valid (&params->expire))
+    expire = g_strdup_printf ("Expire-Date: %04d-%02d-%02d\n",
+                              g_date_get_year (&params->expire),
+                              g_date_get_month (&params->expire),
+                              g_date_get_day (&params->expire));
+
   /* Assemble the final parameter string */
   string = g_strdup_printf ("<GnupgKeyParms format=\"internal\">\n"
                             "Key-Type: %s\n"
                             "Key-Length: %i\n"
+                            "Key-Usage: sign\n"
                             "%s" /* Subkeys */
-                            "Name-Real: %s\n"
-                            "%s" /* Email Address */
+                            "%s" /* Name */
+                            "%s" /* Email */
                             "%s" /* Comment */
                             "%s" /* Expiration date */
-                            "Passphrase: %s\n"
-                            "</GnupgKeyParms>\n", key_algo, params->keysize,
-                            subkeys, params->userID, email, comment, expire,
-                            params->password);
+                            "%%ask-passphrase\n"
+                            "</GnupgKeyParms>\n",
+                            key_algo,
+                            params->keysize,
+                            subkeys? subkeys : "",
+                            name? name:"",
+                            email? email : "",
+                            comment? comment : "",
+                            expire? expire : "");
 
   /* Free auxiliary strings if they are not empty */
-  if (subkeys[0])
-    g_free (subkeys);
-  if (email[0])
-    g_free (email);
-  if (comment[0])
-    g_free (comment);
-  if (expire[0])
-    g_free (expire);
+  g_free (subkeys);
+  g_free (name);
+  g_free (email);
+  g_free (comment);
+  g_free (expire);
 
   return string;
 }
@@ -407,7 +451,7 @@ build_genkey_parms (GPAKeyGenParameters *params)
    the parameters required by GPGME and returns whatever
    gpgme_op_genkey_start returns.  */
 gpg_error_t
-gpa_generate_key_start (gpgme_ctx_t ctx, GPAKeyGenParameters *params)
+gpa_generate_key_start (gpgme_ctx_t ctx, gpa_keygen_para_t *params)
 {
   gchar *parm_string;
   gpg_error_t err;
@@ -420,12 +464,12 @@ gpa_generate_key_start (gpgme_ctx_t ctx, GPAKeyGenParameters *params)
 }
 
 
-/* Retrieve the path to the GnuPG executable.  */
+/* Retrieve the path to the GPG executable.  */
 static const gchar *
 get_gpg_path (void)
 {
   gpgme_engine_info_t engine;
-  
+
   gpgme_get_engine_info (&engine);
   while (engine)
     {
@@ -437,6 +481,77 @@ get_gpg_path (void)
 }
 
 
+/* Retrieve the path to the GPGSM executable.  */
+static const gchar *
+get_gpgsm_path (void)
+{
+  gpgme_engine_info_t engine;
+
+  gpgme_get_engine_info (&engine);
+  while (engine)
+    {
+      if (engine->protocol == GPGME_PROTOCOL_CMS)
+	return engine->file_name;
+      engine = engine->next;
+    }
+  return NULL;
+}
+
+
+/* Retrieve the path to the GPGCONF executable.  */
+static const gchar *
+get_gpgconf_path (void)
+{
+  gpgme_engine_info_t engine;
+
+  gpgme_get_engine_info (&engine);
+  while (engine)
+    {
+      if (engine->protocol == GPGME_PROTOCOL_GPGCONF)
+	return engine->file_name;
+      engine = engine->next;
+    }
+  return NULL;
+}
+
+
+/* Retrieve the path to the GPG-CONNECT-AGENT executable.  Note that
+   the caller must free the returned string.  */
+static char *
+get_gpg_connect_agent_path (void)
+{
+  const char *gpgconf;
+  char *fname, *p;
+
+  gpgconf = get_gpgconf_path ();
+  if (!gpgconf)
+    return NULL;
+
+#ifdef G_OS_WIN32
+# define NEWNAME "gpg-connect-agent.exe"
+#else
+# define NEWNAME "gpg-connect-agent"
+#endif
+
+  fname = g_malloc (strlen (gpgconf) + strlen (NEWNAME) + 1);
+  strcpy (fname, gpgconf);
+#ifdef G_OS_WIN32
+  for (p=fname; *p; p++)
+    if (*p == '\\')
+      *p = '/';
+#endif /*G_OS_WIN32*/
+  p = strrchr (fname, '/');
+  if (p)
+    p++;
+  else
+    p = fname;
+  strcpy (p, NEWNAME);
+
+#undef NEWNAME
+  return fname;
+}
+
+
 /* Backup a key. It exports both the public and secret keys to a file.
    Returns TRUE on success and FALSE on error. It displays errors to
    the user.  */
@@ -447,25 +562,26 @@ gpa_backup_key (const gchar *fpr, const char *filename)
   gchar *err;
   FILE *file;
   gint ret_code;
-  gchar *header_argv[] = 
+  gchar *header_argv[] =
     {
       NULL, "--batch", "--no-tty", "--fingerprint", (gchar*) fpr, NULL
     };
-  gchar *pub_argv[] = 
+  gchar *pub_argv[] =
     {
       NULL, "--batch", "--no-tty", "--armor", "--export", (gchar*) fpr, NULL
     };
-  gchar *sec_argv[] = 
+  gchar *sec_argv[] =
     {
-      NULL, "--batch", "--no-tty", "--armor", "--export-secret-key", 
+      NULL, "--batch", "--no-tty", "--armor", "--export-secret-key",
       (gchar*) fpr, NULL
     };
   const gchar *path;
   mode_t mask;
 
-  /* Get the gpg path */
+  /* Get the gpg path.  */
   path = get_gpg_path ();
-  g_assert (path != NULL);
+  g_return_val_if_fail (path && *path, FALSE);
+
   /* Add the executable to the arg arrays */
   header_argv[0] = (gchar*) path;
   pub_argv[0] = (gchar*) path;
@@ -524,56 +640,28 @@ gpa_backup_key (const gchar *fpr, const char *filename)
 
 
 void
-gpa_key_gen_free_parameters(GPAKeyGenParameters * params)
+gpa_keygen_para_free (gpa_keygen_para_t *params)
 {
-  g_free (params->userID);
-  g_free (params->email);
-  g_free (params->comment);
-  if (params->expiryDate)
-    g_date_free (params->expiryDate);
-  g_free (params);
+  if (params)
+    {
+      g_free (params->name);
+      g_free (params->email);
+      g_free (params->comment);
+      g_free (params->r_error_desc);
+      g_free (params);
+    }
 }
 
 
-GPAKeyGenParameters *
-key_gen_params_new(void)
+gpa_keygen_para_t *
+gpa_keygen_para_new (void)
 {
-  GPAKeyGenParameters * params = g_malloc (sizeof (*params));
-  params->userID = NULL;
-  params->email = NULL;
-  params->comment = NULL;
-  params->expiryDate = NULL;
-  params->interval = 0;
+  gpa_keygen_para_t *params = xcalloc (1, sizeof *params);
+  g_date_clear (&params->expire, 1);
   return params;
 }
 
 
-static gchar *algorithm_strings[] = {
-  N_("DSA and ElGamal (default)"),
-  N_("DSA (sign only)"),
-  N_("RSA (sign only)"),
-};
-
-
-const gchar *
-gpa_algorithm_string (GPAKeyGenAlgo algo)
-{
-  return _(algorithm_strings[algo]);
-}
-
-
-GPAKeyGenAlgo
-gpa_algorithm_from_string (const gchar * string)
-{
-  GPAKeyGenAlgo result;
-
-  result = GPA_KEYGEN_ALGO_FIRST;
-  while (result <= GPA_KEYGEN_ALGO_LAST &&
-	 strcmp (string, _(algorithm_strings[result])) != 0)
-    result++;
-  return result;
-}
-
 
 /* Ownertrust strings.  */
 const gchar *
@@ -582,7 +670,7 @@ gpa_key_ownertrust_string (gpgme_key_t key)
   if (key->protocol == GPGME_PROTOCOL_CMS)
     return "";
 
-  switch (key->owner_trust) 
+  switch (key->owner_trust)
     {
     case GPGME_VALIDITY_UNKNOWN:
     case GPGME_VALIDITY_UNDEFINED:
@@ -611,7 +699,7 @@ gpa_key_validity_string (gpgme_key_t key)
 {
   if (!key->uids)
     return _("Unknown");
-  switch (key->uids->validity) 
+  switch (key->uids->validity)
     {
     case GPGME_VALIDITY_UNKNOWN:
     case GPGME_VALIDITY_UNDEFINED:
@@ -638,7 +726,7 @@ gpa_key_validity_string (gpgme_key_t key)
 
 static GtkWidget *
 passphrase_question_label (const char *uid_hint,
-			   const char *passphrase_info, 
+			   const char *passphrase_info,
 			   int prev_was_bad)
 {
   GtkWidget *label;
@@ -687,7 +775,7 @@ passphrase_question_label (const char *uid_hint,
 /* This is the function called by GPGME when it wants a passphrase.  */
 gpg_error_t
 gpa_passphrase_cb (void *hook, const char *uid_hint,
-		   const char *passphrase_info, 
+		   const char *passphrase_info,
 		   int prev_was_bad, int fd)
 {
   GtkWidget * dialog;
@@ -697,7 +785,7 @@ gpa_passphrase_cb (void *hook, const char *uid_hint,
   GtkWidget * pixmap;
   GtkResponseType response;
   gchar *passphrase;
-  
+
   dialog = gtk_dialog_new_with_buttons (_("Enter Passphrase"),
 					NULL, GTK_DIALOG_MODAL,
                                         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
@@ -709,7 +797,7 @@ gpa_passphrase_cb (void *hook, const char *uid_hint,
                                            -1);
 
   hbox = gtk_hbox_new (FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox, 
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox,
 		      TRUE, FALSE, 10);
   pixmap = gtk_image_new_from_stock (GTK_STOCK_DIALOG_QUESTION,
 				     GTK_ICON_SIZE_DIALOG);
@@ -731,7 +819,7 @@ gpa_passphrase_cb (void *hook, const char *uid_hint,
   /* Run the dialog */
   gtk_widget_show_all (dialog);
   response = gtk_dialog_run (GTK_DIALOG (dialog));
-  passphrase = g_strdup_printf ("%s\n", 
+  passphrase = g_strdup_printf ("%s\n",
 				gtk_entry_get_text (GTK_ENTRY (entry)));
   gtk_widget_destroy (dialog);
   if (response == GTK_RESPONSE_OK)
@@ -743,7 +831,7 @@ gpa_passphrase_cb (void *hook, const char *uid_hint,
       if (WriteFile ((HANDLE) _get_osfhandle (fd), passphrase,
 		     passphrase_len, &res, NULL) == 0
 	  || res < passphrase_len)
-	{	
+	{
 	  g_free (passphrase);
 	  return gpg_error (gpg_err_code_from_errno (EIO));
 	}
@@ -778,16 +866,18 @@ static gchar *
 string_to_utf8 (const gchar *string)
 {
   const char *s;
-  
+
   if (!string)
     return NULL;
-  
+
   /* Due to a bug in old and not so old PGP versions user IDs have
      been copied verbatim into the key.  Thus many users with Umlauts
      et al. in their name will see their names garbled.  Although this
      is not an issue for me (;-)), I have a couple of friends with
      Umlauts in their name, so let's try to make their life easier by
-     detecting invalid encodings and convert that to Latin-1. */
+     detecting invalid encodings and convert that to Latin-1.  We use
+     this even for X.509 because it may make things even better given
+     all the invalid encodings often found in X.509 certificates.  */
   for (s = string; *s && !(*s & 0x80); s++)
     ;
   if (*s && ((s[1] & 0xc0) == 0x80) && ( ((*s & 0xe0) == 0xc0)
@@ -795,7 +885,7 @@ string_to_utf8 (const gchar *string)
                                          || ((*s & 0xf8) == 0xf0)
                                          || ((*s & 0xfc) == 0xf8)
                                          || ((*s & 0xfe) == 0xfc)) )
-    {  
+    {
       /* Possible utf-8 character followed by continuation byte.
          Although this might still be Latin-1 we better assume that it
          is valid utf-8. */
@@ -830,6 +920,10 @@ gpa_gpgme_key_get_userid (gpgme_user_id_t uid)
   /* Tag revoked UID's*/
   if (uid->revoked)
     {
+      /* FIXME: I think, this code should be simply disabled. Even if
+	 the uid U is revoked, it is "U", not "[Revoked] U". Side
+	 note: adding this prefix obviously breaks sorting by
+	 uid. -moritz */
       gchar *tmp = g_strdup_printf ("[%s] %s", _("Revoked"), uid_utf8);
       g_free (uid_utf8);
       uid_utf8 = tmp;
@@ -1039,7 +1133,7 @@ gpa_gpgme_copy_keyarray (gpgme_key_t *keys)
   for (idx=0; keys[idx]; idx++)
     ;
   idx++;
-  newarray = g_new (gpgme_key_t, idx); 
+  newarray = g_new (gpgme_key_t, idx);
   for (idx=0; keys[idx]; idx++)
     {
       gpgme_key_ref (keys[idx]);
@@ -1058,7 +1152,7 @@ gpa_gpgme_release_keyarray (gpgme_key_t *keys)
   if (keys)
     {
       int idx;
-      
+
       for (idx=0; keys[idx]; idx++)
         gpgme_key_unref (keys[idx]);
       g_free (keys);
@@ -1146,7 +1240,7 @@ compare_version_strings (const char *my_version,
 
   if (my_major > rq_major
       || (my_major == rq_major && my_minor > rq_minor)
-      || (my_major == rq_major && my_minor == rq_minor 
+      || (my_major == rq_major && my_minor == rq_minor
 	  && my_micro > rq_micro)
       || (my_major == rq_major && my_minor == rq_minor
 	  && my_micro == rq_micro && strcmp (my_plvl, rq_plvl) >= 0))
@@ -1171,14 +1265,14 @@ gpa_switch_to_gpg2 (void)
   newname = xmalloc (strlen (oldname) + 1 + 1);
   strcpy (newname, oldname);
 #ifdef G_OS_WIN32
-# define OLD_NAME "gpg.exe"  
-# define NEW_NAME "gpg2.exe"  
+# define OLD_NAME "gpg.exe"
+# define NEW_NAME "gpg2.exe"
   for (p=newname; *p; p++)
     if (*p == '\\')
       *p = '/';
 #else
-# define OLD_NAME "gpg"  
-# define NEW_NAME "gpg2"  
+# define OLD_NAME "gpg"
+# define NEW_NAME "gpg2"
 #endif /*G_OS_WIN32*/
   p = strrchr (newname, '/');
   if (p)
@@ -1204,7 +1298,7 @@ int
 is_gpg_version_at_least (const char *need_version)
 {
   gpgme_engine_info_t engine;
-  
+
   gpgme_get_engine_info (&engine);
   while (engine)
     {
@@ -1215,3 +1309,293 @@ is_gpg_version_at_least (const char *need_version)
   return 0; /* No gpg-engine available. */
 }
 
+
+/* Structure used to communicate with gpg_simple_stderr_cb.  */
+struct gpg_simple_stderr_parm_s
+{
+  gboolean (*cb)(void *opaque, char *line);
+  void *cb_arg;
+  GString *string;
+};
+
+/* Helper for gpa_start_simple_gpg_command.  */
+static gboolean
+gpg_simple_stderr_cb (GIOChannel *channel, GIOCondition condition,
+                      void *user_data)
+{
+  struct gpg_simple_stderr_parm_s *parm = user_data;
+  GIOStatus status;
+  char *line, *p;
+
+  if ((condition & G_IO_IN))
+    {
+      /* We don't use a while but an if because that allows to update
+         progress bars nicely.  A bit slower, but no real problem.  */
+      if ((status = g_io_channel_read_line_string
+           (channel, parm->string, NULL, NULL)) == G_IO_STATUS_NORMAL)
+        {
+          line = parm->string->str;
+
+          /* We care only about status lines.  */
+          if (!strncmp (line, "[GNUPG:] ", 9))
+            {
+              line += 9;
+
+              /* Strip line terminator.  */
+              p = strchr (line, '\n');
+              if (p)
+                {
+                  if (p > line && p[-1] == '\r')
+                    p[-1] = 0;
+                  else
+                    *p = 0;
+                }
+
+              /* Call user callback.  */
+              if (parm->cb && !parm->cb (parm->cb_arg, line))
+                {
+                  /* User requested EOF.  */
+                  goto cleanup;
+                }
+            }
+        }
+      if (status != G_IO_STATUS_NORMAL && status != G_IO_STATUS_AGAIN )
+        {
+          /* Error or EOF.  */
+          goto cleanup;
+        }
+    }
+  if ((condition & G_IO_HUP))
+    {
+      goto cleanup;
+    }
+
+  return TRUE;  /* Keep on watching this channel. */
+
+ cleanup:
+  if (parm->cb)
+    parm->cb (parm->cb_arg, NULL);  /* Tell user about EOF.  */
+  g_string_free (parm->string, TRUE);
+  xfree (parm);
+  g_io_channel_unref (channel);  /* Close channel.  */
+  return FALSE;  /* Remove us from the watch list.  */
+}
+
+
+/* Run gpg asynchronously with the given arguments and return a gpg
+   error code on error.  The list of arguments are all of type (const
+   char*) and end with a NULL argument (FIRST_ARG may already be NULL,
+   but that does not make any sense).  STDIN and STDOUT are connected
+   to /dev/null.  No more than 20 arguments may be given.
+
+   Because the protocol GPGME_PROTOCOL_ASSUAN makes no sense here, it
+   is used to call gpg-connect-agent.
+
+   If the function returns success the provided callback CB is called
+   for each line received on STDERR.  EOF is send to this callback by
+   passing a LINE argument of NULL.  The callback may use this for
+   cleanup. If the callback returns FALSE, an EOF is forced with the
+   result that the callback is called once more with LINE set to NULL.  */
+gpg_error_t
+gpa_start_simple_gpg_command (gboolean (*cb)(void *opaque, char *line),
+                              void *cb_arg, gpgme_protocol_t protocol,
+                              const char *first_arg, ...)
+{
+  char *argv[24];
+  int argc;
+  int fd_stderr;
+  GIOChannel *channel;
+  struct gpg_simple_stderr_parm_s *parm = NULL;
+  char *freeme = NULL;
+
+  if (protocol == GPGME_PROTOCOL_OpenPGP)
+    argv[0] = (char*)get_gpg_path ();
+  else if (protocol == GPGME_PROTOCOL_CMS)
+    argv[0] = (char*)get_gpgsm_path ();
+  else if (protocol == GPGME_PROTOCOL_GPGCONF)
+    argv[0] = (char*)get_gpgconf_path ();
+  else if (protocol == GPGME_PROTOCOL_ASSUAN)
+    argv[0] = freeme = get_gpg_connect_agent_path ();
+  else
+    argv[0] = NULL;
+  g_return_val_if_fail (argv[0], gpg_error (GPG_ERR_INV_ARG));
+  argc = 1;
+  if (protocol != GPGME_PROTOCOL_GPGCONF
+      && protocol != GPGME_PROTOCOL_ASSUAN)
+    {
+      argv[argc++] = (char*)"--status-fd";
+      argv[argc++] = (char*)"2";
+    }
+  argv[argc++] = (char*)first_arg;
+  if (first_arg)
+    {
+      va_list arg_ptr;
+      const char *s;
+
+      va_start (arg_ptr, first_arg);
+      while (argc < DIM (argv)-1 && (s=va_arg (arg_ptr, const char *)))
+        argv[argc++] = (char*)s;
+      va_end (arg_ptr);
+      argv[argc] = NULL;
+      g_return_val_if_fail (argc < DIM (argv), gpg_error (GPG_ERR_INV_ARG));
+    }
+
+  parm = g_try_malloc (sizeof *parm);
+  if (!parm)
+    return gpg_error_from_syserror ();
+  parm->cb = cb;
+  parm->cb_arg = cb_arg;
+  parm->string = g_string_sized_new (200);
+
+  if (!g_spawn_async_with_pipes (NULL, argv, NULL,
+                                 (G_SPAWN_STDOUT_TO_DEV_NULL),
+                                 NULL, NULL, NULL,
+                                 NULL, NULL, &fd_stderr, NULL))
+    {
+      gpa_window_error (_("Calling the crypto engine program failed."), NULL);
+      xfree (parm);
+      g_free (freeme);
+      return gpg_error (GPG_ERR_GENERAL);
+    }
+  g_free (freeme);
+#ifdef G_OS_WIN32
+  channel = g_io_channel_win32_new_fd (fd_stderr);
+#else
+  channel = g_io_channel_unix_new (fd_stderr);
+#endif
+  g_io_channel_set_encoding (channel, NULL, NULL);
+  /* Note that we need a buffered channel, so that we can use the read
+     line function.  */
+  g_io_channel_set_close_on_unref (channel, TRUE);
+
+  /* Create a watch for the channel.  */
+  if (!g_io_add_watch (channel, (G_IO_IN|G_IO_HUP),
+                       gpg_simple_stderr_cb, parm))
+    {
+      g_debug ("error creating watch for gpg command");
+      g_io_channel_unref (channel);
+      xfree (parm);
+      return gpg_error (GPG_ERR_GENERAL);
+    }
+
+  return 0;
+}
+
+
+/* Try to start the gpg-agent if it has not yet been started.
+   Starting the agent works in the background.  Thus if the function
+   returns, it is not sure that the agent is now running.  */
+void
+gpa_start_agent (void)
+{
+  gpa_start_simple_gpg_command (NULL, NULL, GPGME_PROTOCOL_ASSUAN,
+                                "NOP", "/bye", NULL);
+}
+
+
+
+/* Fucntions matching the user id verification isn gpg's key generation.  */
+
+const char *
+gpa_validate_gpg_name (const char *name)
+{
+  const char *result = NULL;
+
+  if (!name || !*name)
+    result = _("You must enter a name.");
+  else if (strpbrk (name, "<>"))
+    result = _("Invalid character in name.");
+  else if (g_ascii_isdigit (*name))
+    result = _("Name may not start with a digit.");
+  else if (g_utf8_strlen (name, -1) < 5)
+    result = _("Name is too short.");
+
+  return result;
+}
+
+
+/* Check whether the string has characters not valid in an RFC-822
+   address.  To cope with OpenPGP we allow non-ascii characters
+   so that for example umlauts are legal in an email address.  An
+   OpenPGP user ID must be utf-8 encoded but there is no strict
+   requirement for RFC-822.  Thus to avoid IDNA encoding we put the
+   address verbatim as utf-8 into the user ID under the assumption
+   that mail programs handle IDNA at a lower level and take OpenPGP
+   user IDs as utf-8.  */
+static int
+has_invalid_email_chars (const char *s)
+{
+  int at_seen = 0;
+  const char *valid_chars=
+    "01234567890_-.abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
+
+  for ( ; *s; s++ )
+    {
+      if ((*s & 0x80))
+        continue; /* We only care about ASCII.  */
+      if (*s == '@')
+        at_seen=1;
+      else if (!at_seen && !( !!strchr( valid_chars, *s ) || *s == '+' ))
+        return 1;
+      else if (at_seen && !strchr (valid_chars, *s))
+        return 1;
+    }
+  return 0;
+}
+
+
+static int
+string_count_chr (const char *string, int c)
+{
+  int count;
+
+  for (count=0; *string; string++ )
+    if ( *string == c )
+      count++;
+  return count;
+}
+
+
+/* Check whether NAME represents a valid mailbox according to RFC822
+   except for non-ascii utf-8 characters. Returns true if so. */
+static int
+is_valid_mailbox (const char *name)
+{
+  return !( !name
+            || !*name
+            || has_invalid_email_chars (name)
+            || string_count_chr (name,'@') != 1
+            || *name == '@'
+            || name[strlen(name)-1] == '@'
+            || name[strlen(name)-1] == '.'
+            || strstr (name, "..") );
+}
+
+
+const char *
+gpa_validate_gpg_email (const char *email)
+{
+  const char *result = NULL;
+
+  if (!email || !*email)
+    ;
+  else if (!is_valid_mailbox (email))
+    result = _("Email address is not valid.");
+
+  return result;
+}
+
+
+const char *
+gpa_validate_gpg_comment (const char *comment)
+{
+  const char *result = NULL;
+
+  if (!comment || !*comment)
+    ;
+  else if (strpbrk (comment, "()"))
+    result = _("Invalid character in comments.");
+
+  return result;
+}
+
