@@ -26,25 +26,29 @@
 #include <gtk/gtk.h>
 #include <gpgme.h>
 
+/* Internal algorithm identifiers, describing which keys to
+   create.  */
 typedef enum
   {
+    GPA_KEYGEN_ALGO_RSA_RSA,
+    GPA_KEYGEN_ALGO_RSA_ELGAMAL,
+    GPA_KEYGEN_ALGO_RSA,
     GPA_KEYGEN_ALGO_DSA_ELGAMAL,
     GPA_KEYGEN_ALGO_DSA,
-    GPA_KEYGEN_ALGO_RSA
-  } GPAKeyGenAlgo;
+    GPA_KEYGEN_ALGO_VIA_CARD
+  } gpa_keygen_algo_t;
+
 
-#define GPA_KEYGEN_ALGO_FIRST GPA_KEYGEN_ALGO_DSA_ELGAMAL
-#define GPA_KEYGEN_ALGO_LAST GPA_KEYGEN_ALGO_RSA
 
 typedef struct
- {
+{
   /* User ID.  */
-  gchar *userID;
+  gchar *name;
   gchar *email;
   gchar *comment;
-
+  
   /* Algorithm.  */
-  GPAKeyGenAlgo algo;
+  gpa_keygen_algo_t algo;
 
   /* Key size.  */
   gint keysize;
@@ -52,14 +56,17 @@ typedef struct
   /* The password to use.  */
   gchar *password;
 
-  /* The expiry date.  If expiryDate is not NULL it holds the expiry
-     date, otherwise if interval is not zero, it defines the period of
-     time until expiration together with unit (which is one of d, w,
-     m, y), otherwise the user chose "never expire".  */
-  GDate *expiryDate;
-  gint interval;
-  gchar unit;
-} GPAKeyGenParameters;
+  /* Epiration date.  It is only used if it is valid.  */
+  GDate expire;
+
+  /* True if the encryption key is created with an backup file.  */
+  gboolean backup;
+
+  /* Used to return an error description.  */
+  char *r_error_desc;
+
+} gpa_keygen_para_t;
+
 
 /* Report an unexpected error in GPGME and quit the application.
    Better to use the macro instead of the function.  */
@@ -68,6 +75,7 @@ typedef struct
 void _gpa_gpgme_error (gpg_error_t err, const char *file, int line);
 
 /* The same as gpa_gpgme_error, without quitting.  */
+void gpa_gpgme_warning_ext (gpg_error_t err, const char *desc);
 void gpa_gpgme_warning (gpg_error_t err);
 
 /* Initialize a gpgme_ctx_t for use with GPA.  */
@@ -116,20 +124,16 @@ void dump_data_to_clipboard (gpgme_data_t data, GtkClipboard *clipboard);
    the parameters required by Gpgme and returns whatever
    gpgme_op_genkey_start returns.  */
 gpg_error_t gpa_generate_key_start (gpgme_ctx_t ctx, 
-				    GPAKeyGenParameters *params);
+				    gpa_keygen_para_t *params);
 
 /* Backup a key.  It exports both the public and secret keys to a
    file.  Returns TRUE on success and FALSE on error.  It displays
    errors to the user.  */
 gboolean gpa_backup_key (const gchar *fpr, const char *filename);
 
-GPAKeyGenParameters *key_gen_params_new (void);
+gpa_keygen_para_t *gpa_keygen_para_new (void);
 
-void gpa_key_gen_free_parameters (GPAKeyGenParameters *params);
-
-const gchar * gpa_algorithm_string (GPAKeyGenAlgo algo);
-
-GPAKeyGenAlgo gpa_algorithm_from_string (const gchar * string);
+void gpa_keygen_para_free (gpa_keygen_para_t *params);
 
 /* Ownertrust strings.  */
 const gchar *gpa_key_ownertrust_string (gpgme_key_t key);
@@ -194,5 +198,20 @@ void gpa_switch_to_gpg2 (void);
 /* Return true if the gpg engine has at least version NEED_VERSION.  */
 int is_gpg_version_at_least (const char *need_version);
 
+/* Run a simple gpg command.  */
+gpg_error_t gpa_start_simple_gpg_command (gboolean (*cb)
+                                          (void *opaque, char *line),
+                                          void *cb_arg, 
+                                          gpgme_protocol_t protocol,
+                                          const char *first_arg, ...
+                                          ) G_GNUC_NULL_TERMINATED;
+
+void gpa_start_agent (void);
+
+/* Funtions to check user inputs in the same way gpg does.  */
+const char *gpa_validate_gpg_name (const char *name);
+const char *gpa_validate_gpg_email (const char *email);
+const char *gpa_validate_gpg_comment (const char *comment);
+
 
 #endif /*GPGMETOOLS_H*/
