@@ -64,7 +64,7 @@ struct conn_ctrl_s
      still registyered.  */
   int client_died;
 
-  /* This is a helper to detect that the unfinished erroe code actually
+  /* This is a helper to detect that the unfinished error code actually
      comes from our command handler.  */
   int is_unfinished;
 
@@ -96,6 +96,7 @@ struct conn_ctrl_s
      the sender ist just informational. */
   gchar *sender;
   int sender_just_info;
+  gpgme_protocol_t sender_protocol_hint;
 
   /* Session information:  A session number and a malloced title or NULL.  */
   unsigned int session_number;
@@ -551,23 +552,23 @@ prepare_io_streams (assuan_context_t ctx,
 
 
 
-/* SESSION <number> [<string>]
-
-   The NUMBER is an arbitrary value, a server may use to associate
-   simultaneous running sessions.  It is a 32 bit unsigned integer
-   with 0 as a special value indicating that no session association
-   shall be done.
-
-   If STRING is given, the server may use this as the title of a
-   window or, in the case of an email operation, to extract the
-   sender's address. The string may contain spaces; thus no
-   plus-escaping is used.
-
-   This command may be used at any time and overrides the effect of
-   the last command.  A RESET command undoes the effect of this
-   command.
-*/
-static int
+static const char hlp_session[] = 
+  "SESSION <number> [<string>]\n"
+  "\n"
+  "The NUMBER is an arbitrary value, a server may use to associate\n"
+  "simultaneous running sessions.  It is a 32 bit unsigned integer\n"
+  "with 0 as a special value indicating that no session association\n"
+  "shall be done.\n"
+  "\n"
+  "If STRING is given, the server may use this as the title of a\n"
+  "window or, in the case of an email operation, to extract the\n"
+  "sender's address. The string may contain spaces; thus no\n"
+  "plus-escaping is used.\n"
+  "\n"
+  "This command may be used at any time and overrides the effect of\n"
+  "the last command.  A RESET command undoes the effect of this\n"
+  "command.";
+static gpg_error_t
 cmd_session (assuan_context_t ctx, char *line)
 {
   conn_ctrl_t ctrl = assuan_get_pointer (ctx);
@@ -586,14 +587,15 @@ cmd_session (assuan_context_t ctx, char *line)
 
 
 
-/*  RECIPIENT <recipient>
-
-  Set the recipient for the encryption.  <recipient> is an RFC2822
-  recipient name.  This command may or may not check the recipient for
-  validity right away; if it does not (as here) all recipients are
-  checked at the time of the ENCRYPT command.  All RECIPIENT commands
-  are cumulative until a RESET or an successful ENCRYPT command.  */
-static int
+static const char hlp_recipient[] = 
+  "RECIPIENT <recipient>\n"
+  "\n"
+  "Set the recipient for the encryption.  <recipient> is an RFC2822\n"
+  "recipient name.  This command may or may not check the recipient for\n"
+  "validity right away; if it does not (as here) all recipients are\n"
+  "checked at the time of the ENCRYPT command.  All RECIPIENT commands\n"
+  "are cumulative until a RESET or an successful ENCRYPT command.";
+static gpg_error_t
 cmd_recipient (assuan_context_t ctx, char *line)
 {
   conn_ctrl_t ctrl = assuan_get_pointer (ctx);
@@ -609,12 +611,11 @@ cmd_recipient (assuan_context_t ctx, char *line)
 
 
 
-/* MESSAGE FD=<n>
-
-   Set the file descriptor to read a message which is used with
-   detached signatures.
- */
-static int 
+static const char hlp_message[] = 
+  "MESSAGE FD[=<n>]\n"
+  "\n"
+  "Set the file descriptor to read a message of a detached signature to N.";
+static gpg_error_t
 cmd_message (assuan_context_t ctx, char *line)
 {
   conn_ctrl_t ctrl = assuan_get_pointer (ctx);
@@ -644,7 +645,7 @@ cont_encrypt (assuan_context_t ctx, gpg_error_t err)
 {
   conn_ctrl_t ctrl = assuan_get_pointer (ctx);
 
-  g_debug ("cont_encrypt called with with ERR=%s <%s>",
+  g_debug ("cont_encrypt called with ERR=%s <%s>",
            gpg_strerror (err), gpg_strsource (err));
 
   finish_io_streams (ctx, NULL, NULL, NULL);
@@ -654,11 +655,11 @@ cont_encrypt (assuan_context_t ctx, gpg_error_t err)
 }
 
 
-/* ENCRYPT --protocol=OpenPGP|CMS
-
-   Encrypt the data received on INPUT to OUTPUT.
-*/
-static int 
+static const char hlp_encrypt[] = 
+  "ENCRYPT --protocol=OpenPGP|CMS\n"
+  "\n"
+  "Encrypt the data received on INPUT to OUTPUT.";
+static gpg_error_t
 cmd_encrypt (assuan_context_t ctx, char *line)
 {
   conn_ctrl_t ctrl = assuan_get_pointer (ctx);
@@ -726,7 +727,7 @@ cont_prep_encrypt (assuan_context_t ctx, gpg_error_t err)
 {
   conn_ctrl_t ctrl = assuan_get_pointer (ctx);
 
-  g_debug ("cont_prep_encrypt called with with ERR=%s <%s>",
+  g_debug ("cont_prep_encrypt called with ERR=%s <%s>",
            gpg_strerror (err), gpg_strsource (err));
 
   if (!err)
@@ -750,11 +751,12 @@ cont_prep_encrypt (assuan_context_t ctx, gpg_error_t err)
   assuan_process_done (ctx, err);
 }
 
-/* PREP_ENCRYPT [--protocol=OpenPGP|CMS]
-
-   Dummy encryption command used to check whether the given recipients
-   are all valid and to tell the client the preferred protocol.  */
-static int 
+static const char hlp_prep_encrypt[] = 
+  "PREP_ENCRYPT [--protocol=OpenPGP|CMS]\n"
+  "\n"
+  "Dummy encryption command used to check whether the given recipients\n"
+  "are all valid and to tell the client the preferred protocol.";
+static gpg_error_t
 cmd_prep_encrypt (assuan_context_t ctx, char *line)
 {
   conn_ctrl_t ctrl = assuan_get_pointer (ctx);
@@ -805,28 +807,39 @@ cmd_prep_encrypt (assuan_context_t ctx, char *line)
 
 
 
-/*  SENDER <email>
-
-    EMAIL is the plain ASCII encoded address ("addr-spec" as per
-    RFC-2822) enclosed in angle brackets.  The address set by this
-    command is valid until a successful @code{SIGN} command or until a
-    @code{RESET} command.  A second command overrides the effect of
-    the first one; if EMAIL is not given the server shall use the
-    default signing key.  */
-static int
+static const char hlp_sender[] =
+  "SENDER <email>\n"
+  "\n"
+  "EMAIL is the plain ASCII encoded address (\"addr-spec\" as per\n"
+  "RFC-2822) enclosed in angle brackets.  The address set by this\n"
+  "command is valid until a successful \"SIGN\" command or until a\n"
+  "\"RESET\" command.  A second command overrides the effect of\n"
+  "the first one; if EMAIL is not given the server shall use the\n"
+  "default signing key.";
+static gpg_error_t
 cmd_sender (assuan_context_t ctx, char *line)
 {
   conn_ctrl_t ctrl = assuan_get_pointer (ctx);
-  gpg_error_t err = 0;
+  gpg_error_t err;
+  gpgme_protocol_t protocol;
+
+  err = parse_protocol_option (ctx, line, 0, &protocol);
+  if (err)
+    goto leave;
 
   ctrl->sender_just_info = has_option (line, "--info");
+
   line = skip_options (line);
 
   xfree (ctrl->sender);
   ctrl->sender = NULL;
   if (*line)
     ctrl->sender = xstrdup (line);
+  
+  if (!err)
+    ctrl->sender_protocol_hint = protocol;
 
+ leave:
   return assuan_process_done (ctx, err);
 }
 
@@ -838,7 +851,7 @@ cont_sign (assuan_context_t ctx, gpg_error_t err)
 {
   conn_ctrl_t ctrl = assuan_get_pointer (ctx);
 
-  g_debug ("cont_sign called with with ERR=%s <%s>",
+  g_debug ("cont_sign called with ERR=%s <%s>",
            gpg_strerror (err), gpg_strsource (err));
 
   finish_io_streams (ctx, NULL, NULL, NULL);
@@ -846,16 +859,17 @@ cont_sign (assuan_context_t ctx, gpg_error_t err)
     {
       xfree (ctrl->sender);
       ctrl->sender = NULL;
+      ctrl->sender_protocol_hint = GPGME_PROTOCOL_UNKNOWN;
     }
   assuan_process_done (ctx, err);
 }
 
 
-/* SIGN --protocol=OpenPGP|CMS [--detached]
-
-   Sign the data received on INPUT to OUTPUT.
-*/
-static int 
+static const char hlp_sign[] =
+  "SIGN --protocol=OpenPGP|CMS [--detached]\n"
+  "\n"
+  "Sign the data received on INPUT to OUTPUT.";
+static gpg_error_t
 cmd_sign (assuan_context_t ctx, char *line)
 {
   conn_ctrl_t ctrl = assuan_get_pointer (ctx);
@@ -915,7 +929,7 @@ cmd_sign (assuan_context_t ctx, char *line)
 static void
 cont_decrypt (assuan_context_t ctx, gpg_error_t err)
 {
-  g_debug ("cont_decrypt called with with ERR=%s <%s>",
+  g_debug ("cont_decrypt called with ERR=%s <%s>",
            gpg_strerror (err), gpg_strsource (err));
 
   finish_io_streams (ctx, NULL, NULL, NULL);
@@ -923,15 +937,16 @@ cont_decrypt (assuan_context_t ctx, gpg_error_t err)
 }
 
 
-/* DECRYPT --protocol=OpenPGP|CMS [--no-verify]
-
-   Decrypt a message given by the source set with the INPUT command
-   and write the plaintext to the sink set with the OUTPUT command.
-
-   If the option --no-verify is given, the server should not try to
-   verify a signature, in case the input data is an OpenPGP combined
-   message.  */
-static int 
+static const char hlp_decrypt[] = 
+  "DECRYPT --protocol=OpenPGP|CMS [--no-verify]\n"
+  "\n"
+  "Decrypt a message given by the source set with the INPUT command\n"
+  "and write the plaintext to the sink set with the OUTPUT command.\n"
+  "\n"
+  "If the option --no-verify is given, the server should not try to\n"
+  "verify a signature, in case the input data is an OpenPGP combined\n"
+  "message.";
+static gpg_error_t
 cmd_decrypt (assuan_context_t ctx, char *line)
 {
   conn_ctrl_t ctrl = assuan_get_pointer (ctx);
@@ -995,7 +1010,7 @@ cmd_decrypt (assuan_context_t ctx, char *line)
 static void
 cont_verify (assuan_context_t ctx, gpg_error_t err)
 {
-  g_debug ("cont_verify called with with ERR=%s <%s>",
+  g_debug ("cont_verify called with ERR=%s <%s>",
            gpg_strerror (err), gpg_strsource (err));
 
   finish_io_streams (ctx, NULL, NULL, NULL);
@@ -1003,50 +1018,50 @@ cont_verify (assuan_context_t ctx, gpg_error_t err)
 }
 
 
-/* VERIFY --protocol=OpenPGP|CMS [--silent]
-
-   Verify a message.  Depending on the combination of the
-   sources/sinks set by the commands MESSAGE, INPUT and OUTPUT, the
-   verification mode is selected like this:
-
-   MESSAGE and INPUT
-     This indicates a detached signature.  Output data is not applicable.
-
-   INPUT 
-     This indicates an opaque signature.  As no output command has
-     been given, we are only required to check the signature.
-
-   INPUT and OUTPUT
-     This indicates an opaque signature.  We write the signed data to
-     the file descriptor set by the OUTPUT command.  This data is even
-     written if the signature can't be verified.
-
-   With the option --silent we won't display any dialog; this is for
-   example used by the client to get the content of an opaque signed
-   message.  Sending the follwoing status message is mandatory before
-   sending the OK response:
-
-      SIGSTATUS <flag> <displaystring>
-
-   Defines FLAGS are:
-
-      none
-        The message has a signature but it could not not be verified
-        due to a missing key.
-
-      green
-        The signature is fully valid.
-
-      yellow
-        The signature is valid but additional information was shown
-        regarding the validity of the key.
-
-      red
-        The signature is not valid. 
-
-   The DISPLAYSTRING is a percent-and-plus-encoded string with a short
-   human readable description of the status.  */
-static int 
+static const char hlp_verify[] = 
+  "VERIFY --protocol=OpenPGP|CMS [--silent]\n"
+  "\n"
+  "Verify a message.  Depending on the combination of the\n"
+  "sources/sinks set by the commands MESSAGE, INPUT and OUTPUT, the\n"
+  "verification mode is selected like this:\n"
+  "\n"
+  "MESSAGE and INPUT\n"
+  "  This indicates a detached signature.  Output data is not applicable.\n"
+  "\n"
+  "INPUT \n"
+  "  This indicates an opaque signature.  As no output command has\n"
+  "  been given, we are only required to check the signature.\n"
+  "\n"
+  "INPUT and OUTPUT\n"
+  "  This indicates an opaque signature.  We write the signed data to\n"
+  "  the file descriptor set by the OUTPUT command.  This data is even\n"
+  "  written if the signature can't be verified.\n"
+  "\n"
+  "With the option --silent we won't display any dialog; this is for\n"
+  "example used by the client to get the content of an opaque signed\n"
+  "message.  This status message is always send before an OK response:\n"
+  "\n"
+  "   SIGSTATUS <flag> <displaystring>\n"
+  "\n"
+  "Defined FLAGS are:\n"
+  "\n"
+  "   none\n"
+  "     The message has a signature but it could not not be verified\n"
+  "     due to a missing key.\n"
+  "\n"
+  "   green\n"
+  "     The signature is fully valid.\n"
+  "\n"
+  "   yellow\n"
+  "     The signature is valid but additional information was shown\n"
+  "     regarding the validity of the key.\n"
+  "\n"
+  "   red\n"
+  "     The signature is not valid. \n"
+  "\n"
+  "The DISPLAYSTRING is a percent-and-plus-encoded string with a short\n"
+  "human readable description of the status.";
+static gpg_error_t
 cmd_verify (assuan_context_t ctx, char *line)
 {
   conn_ctrl_t ctrl = assuan_get_pointer (ctx);
@@ -1125,29 +1140,43 @@ cmd_verify (assuan_context_t ctx, char *line)
 
 
 
-/* START_KEYMANAGER
-
-   Pop up the key manager window.  The client expects that the key
-   manager is brought into the foregound and that this command
-   immediatley returns.
-*/
-static int
+static const char hlp_start_keymanager[] = 
+  "START_KEYMANAGER\n"
+  "\n"
+  "Pop up the key manager window.  The client expects that the key\n"
+  "manager is brought into the foregound and that this command\n"
+  "immediatley returns.";
+static gpg_error_t
 cmd_start_keymanager (assuan_context_t ctx, char *line)
 {
-  gpa_open_keyring_editor (NULL, NULL);
+  gpa_open_key_manager (NULL, NULL);
 
   return assuan_process_done (ctx, 0);
 }
 
 
-
-/* START_CONFDIALOG
+static const char hlp_start_cardmanager[] = 
+  "START_CARDMANAGER\n"
+  "\n"
+  "Pop up the card manager window.  The client expects that the key\n"
+  "manager is brought into the foregound and that this command\n"
+  "immediatley returns.";
+static gpg_error_t
+cmd_start_cardmanager (assuan_context_t ctx, char *line)
+{
+  gpa_open_cardmanager (NULL, NULL);
 
-   Pop up the configure dialog.  The client expects that the key
-   manager is brought into the foregound and that this command
-   immediatley returns.
-*/
-static int
+  return assuan_process_done (ctx, 0);
+}
+
+
+static const char hlp_start_confdialog[] = 
+  "START_CONFDIALOG\n"
+  "\n"
+  "Pop up the configure dialog.  The client expects that the key\n"
+  "manager is brought into the foregound and that this command\n"
+  "immediatley returns.";
+static gpg_error_t
 cmd_start_confdialog (assuan_context_t ctx, char *line)
 {
   gpa_open_settings_dialog (NULL, NULL);
@@ -1158,15 +1187,15 @@ cmd_start_confdialog (assuan_context_t ctx, char *line)
 
 
 
-/* GETINFO <what>
-
-   Multipurpose function to return a variety of information.
-   Supported values for WHAT are:
-
-     version     - Return the version of the program.
-     pid         - Return the process id of the server.
- */
-static int
+static const char hlp_getinfo[] = 
+  "GETINFO <what>\n"
+  "\n"
+  "Multipurpose function to return a variety of information.\n"
+  "Supported values for WHAT are:\n"
+  "\n"
+  "  version     - Return the version of the program.\n"
+  "  pid         - Return the process id of the server.";
+static gpg_error_t
 cmd_getinfo (assuan_context_t ctx, char *line)
 {
   gpg_error_t err;
@@ -1270,7 +1299,7 @@ decode_percent_string (char *str)
 
    Set the files on which to operate.
  */
-static int
+static gpg_error_t
 cmd_file (assuan_context_t ctx, char *line)
 {
   gpg_error_t err = 0;
@@ -1341,7 +1370,7 @@ impl_encrypt_sign_files (assuan_context_t ctx, int encrypt, int sign)
 
 
 /* ENCRYPT_FILES --nohup  */
-static int
+static gpg_error_t
 cmd_encrypt_files (assuan_context_t ctx, char *line)
 {
   gpg_error_t err;
@@ -1364,7 +1393,7 @@ cmd_encrypt_files (assuan_context_t ctx, char *line)
 
 
 /* SIGN_FILES --nohup  */
-static int
+static gpg_error_t
 cmd_sign_files (assuan_context_t ctx, char *line)
 {
   gpg_error_t err;
@@ -1387,7 +1416,7 @@ cmd_sign_files (assuan_context_t ctx, char *line)
 
 
 /* ENCRYPT_SIGN_FILES --nohup  */
-static int
+static gpg_error_t
 cmd_encrypt_sign_files (assuan_context_t ctx, char *line)
 {
   gpg_error_t err;
@@ -1449,7 +1478,7 @@ impl_decrypt_verify_files (assuan_context_t ctx, int decrypt, int verify)
 
 
 /* DECRYPT_FILES --nohup  */
-static int
+static gpg_error_t
 cmd_decrypt_files (assuan_context_t ctx, char *line)
 {
   gpg_error_t err;
@@ -1472,7 +1501,7 @@ cmd_decrypt_files (assuan_context_t ctx, char *line)
 
 
 /* VERIFY_FILES --nohup  */
-static int
+static gpg_error_t
 cmd_verify_files (assuan_context_t ctx, char *line)
 {
   gpg_error_t err;
@@ -1495,7 +1524,7 @@ cmd_verify_files (assuan_context_t ctx, char *line)
 
 
 /* DECRYPT_VERIFY_FILES --nohup  */
-static int
+static gpg_error_t
 cmd_decrypt_verify_files (assuan_context_t ctx, char *line)
 {
   gpg_error_t err;
@@ -1518,7 +1547,7 @@ cmd_decrypt_verify_files (assuan_context_t ctx, char *line)
 
 
 /* IMPORT_FILES --nohup  */
-static int
+static gpg_error_t
 cmd_import_files (assuan_context_t ctx, char *line)
 {
   gpg_error_t err;
@@ -1542,7 +1571,7 @@ cmd_import_files (assuan_context_t ctx, char *line)
 
 
 /* CHECKSUM_CREATE_FILES --nohup  */
-static int
+static gpg_error_t
 cmd_checksum_create_files (assuan_context_t ctx, char *line)
 {
   gpg_error_t err;
@@ -1566,7 +1595,7 @@ cmd_checksum_create_files (assuan_context_t ctx, char *line)
 
 
 /* CHECKSUM_VERIFY_FILES --nohup  */
-static int
+static gpg_error_t
 cmd_checksum_verify_files (assuan_context_t ctx, char *line)
 {
   gpg_error_t err;
@@ -1589,8 +1618,8 @@ cmd_checksum_verify_files (assuan_context_t ctx, char *line)
 }
 
 
-static void
-reset_notify (assuan_context_t ctx)
+static gpg_error_t
+reset_notify (assuan_context_t ctx, char *line)
 {
   conn_ctrl_t ctrl = assuan_get_pointer (ctx);
 
@@ -1599,6 +1628,7 @@ reset_notify (assuan_context_t ctx)
   release_files (ctrl);
   xfree (ctrl->sender);
   ctrl->sender = NULL;
+  ctrl->sender_protocol_hint = GPGME_PROTOCOL_UNKNOWN;
   finish_io_streams (ctx, NULL, NULL, NULL);
   close_message_fd (ctrl);
   assuan_close_input_fd (ctx);
@@ -1613,6 +1643,7 @@ reset_notify (assuan_context_t ctx)
   ctrl->session_number = 0;
   xfree (ctrl->session_title);
   ctrl->session_title = NULL;
+  return 0;
 }
 
 
@@ -1622,22 +1653,24 @@ register_commands (assuan_context_t ctx)
 {
   static struct {
     const char *name;
-    int (*handler)(assuan_context_t, char *line);
+    assuan_handler_t handler;
+    const char * const help;
   } table[] = {
-    { "SESSION", cmd_session },
-    { "RECIPIENT", cmd_recipient },
+    { "SESSION", cmd_session, hlp_session },
+    { "RECIPIENT", cmd_recipient, hlp_recipient },
     { "INPUT", NULL },
     { "OUTPUT", NULL },
-    { "MESSAGE", cmd_message },
-    { "ENCRYPT", cmd_encrypt },
-    { "PREP_ENCRYPT", cmd_prep_encrypt },
-    { "SENDER", cmd_sender },
-    { "SIGN", cmd_sign   },
-    { "DECRYPT", cmd_decrypt },
-    { "VERIFY", cmd_verify },
-    { "START_KEYMANAGER", cmd_start_keymanager },
-    { "START_CONFDIALOG", cmd_start_confdialog },
-    { "GETINFO", cmd_getinfo },
+    { "MESSAGE", cmd_message, hlp_message },
+    { "ENCRYPT", cmd_encrypt, hlp_encrypt },
+    { "PREP_ENCRYPT", cmd_prep_encrypt, hlp_prep_encrypt },
+    { "SENDER", cmd_sender, hlp_sender },
+    { "SIGN", cmd_sign, hlp_sign },
+    { "DECRYPT", cmd_decrypt, hlp_decrypt },
+    { "VERIFY", cmd_verify, hlp_verify },
+    { "START_KEYMANAGER", cmd_start_keymanager, hlp_start_keymanager },
+    { "START_CONFDIALOG", cmd_start_confdialog, hlp_start_confdialog },
+    { "START_CARDMANAGER", cmd_start_cardmanager, hlp_start_cardmanager },
+    { "GETINFO", cmd_getinfo, hlp_getinfo },
     { "FILE", cmd_file },
     { "ENCRYPT_FILES", cmd_encrypt_files },
     { "SIGN_FILES", cmd_sign_files },
@@ -1654,7 +1687,8 @@ register_commands (assuan_context_t ctx)
 
   for (i=0; table[i].name; i++)
     {
-      rc = assuan_register_command (ctx, table[i].name, table[i].handler);
+      rc = assuan_register_command (ctx, table[i].name, table[i].handler,
+				    table[i].help);
       if (rc)
         return rc;
     } 
@@ -1665,7 +1699,7 @@ register_commands (assuan_context_t ctx)
 
 /* Prepare for a new connection on descriptor FD.  */
 static assuan_context_t
-connection_startup (int fd)
+connection_startup (assuan_fd_t fd)
 {
   gpg_error_t err;
   assuan_context_t ctx;
@@ -1673,7 +1707,16 @@ connection_startup (int fd)
 
   /* Get an Assuan context for the already accepted file descriptor
      FD.  Allow descriptor passing.  */
-  err = assuan_init_socket_server_ext (&ctx, ASSUAN_INT2FD(fd), 1|2);
+  err = assuan_new (&ctx);
+  if (err)
+    {
+      g_debug ("failed to initialize the new connection: %s",
+               gpg_strerror (err));
+      return NULL;
+    }
+
+  err = assuan_init_socket_server (ctx, fd, (ASSUAN_SOCKET_SERVER_FDPASSING
+                                             | ASSUAN_SOCKET_SERVER_ACCEPTED));
   if (err)
     {
       g_debug ("failed to initialize the new connection: %s",
@@ -1685,7 +1728,7 @@ connection_startup (int fd)
     {
       g_debug ("failed to register commands with Assuan: %s",
                gpg_strerror (err));
-      assuan_deinit_server (ctx);
+      assuan_release (ctx);
       return NULL;
     }
 
@@ -1709,8 +1752,8 @@ connection_finish (assuan_context_t ctx)
     {
       conn_ctrl_t ctrl = assuan_get_pointer (ctx);
 
-      reset_notify (ctx);
-      assuan_deinit_server (ctx);
+      reset_notify (ctx, NULL);
+      assuan_release (ctx);
       g_free (ctrl);
       connection_counter--;
       if (!connection_counter && shutdown_pending)
@@ -1778,14 +1821,23 @@ receive_cb (GIOChannel *channel, GIOCondition condition, void *data)
         }
       else
         {
+	  int done = 0;
           ctrl->in_command++;
-          err = assuan_process_next (ctx);
+          err = assuan_process_next (ctx, &done);
           ctrl->in_command--;
-          g_debug ("assuan_process_next returned: %s",
-                   err == -1? "EOF": gpg_strerror (err));
+	  if (err)
+	    {
+	      g_debug ("assuan_process_next returned: %s <%s>",
+		       gpg_strerror (err), gpg_strsource (err));
+	    }
+	  else
+	    {
+	      g_debug ("assuan_process_next returned: %s",
+		       done ? "done" : "success");
+	    }
           if (gpg_err_code (err) == GPG_ERR_EAGAIN)
             ; /* Ignore.  */
-          else if (gpg_err_code (err) == GPG_ERR_EOF || err == -1)
+          else if (!err && done)
             {
               if (ctrl->cont_cmd)
                 ctrl->client_died = 1; /* Need to delay the cleanup.  */
@@ -1841,14 +1893,14 @@ accept_connection_cb (GIOChannel *listen_channel,
       g_debug ("error accepting connection: %s", strerror (errno));
       goto leave;
     }
-  if (assuan_sock_check_nonce (ASSUAN_INT2FD(fd), &socket_nonce))
+  if (assuan_sock_check_nonce ((assuan_fd_t) fd, &socket_nonce))
     {
       g_debug ("new connection at fd %d refused", fd); 
       goto leave;
     }
 
   g_debug ("new connection at fd %d", fd);
-  ctx = connection_startup (fd);
+  ctx = connection_startup ((assuan_fd_t) fd);
   if (!ctx)
     goto leave;
 
@@ -1884,7 +1936,7 @@ accept_connection_cb (GIOChannel *listen_channel,
 
  leave:
   if (fd != -1)
-    assuan_sock_close (ASSUAN_INT2FD (fd));
+    assuan_sock_close ((assuan_fd_t) fd);
   return TRUE; /* Keep the listen_fd in the event loop.  */
 }
 
@@ -1902,7 +1954,7 @@ gpa_start_server (void)
   GIOChannel *channel;
   unsigned int source_id;
 
-  assuan_set_assuan_err_source (GPG_ERR_SOURCE_DEFAULT);
+  assuan_set_gpg_err_source (GPG_ERR_SOURCE_DEFAULT);
   
   socket_name = g_build_filename (gnupg_homedir, "S.uiserver", NULL);
   if (strlen (socket_name)+1 >= sizeof serv_addr.sun_path ) 
@@ -1947,14 +1999,14 @@ gpa_start_server (void)
   g_free (socket_name);
   socket_name = NULL;
 
-  if (listen (ASSUAN_FD2INT (fd), 5) == -1)
+  if (listen ((int) fd, 5) == -1)
     {
       g_debug ("listen() failed: %s\n", strerror (errno));
       assuan_sock_close (fd);
       return;
     }
 #ifdef HAVE_W32_SYSTEM
-  channel = g_io_channel_win32_new_socket (ASSUAN_FD2INT(fd));
+  channel = g_io_channel_win32_new_socket ((int) fd);
 #else
   channel = g_io_channel_unix_new (fd);
 #endif
